"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Generator = void 0;
const random_js_1 = require("random-js");
class Generator {
    constructor(dbConnector, schema, logger) {
        this.dbConnector = dbConnector;
        this.schema = schema;
        this.logger = logger;
        this.continue = false;
        if (schema.settings.seed) {
            this.random = new random_js_1.Random(random_js_1.MersenneTwister19937.seed(schema.settings.seed));
        }
        else {
            this.random = new random_js_1.Random(random_js_1.MersenneTwister19937.autoSeed());
        }
    }
    gotoNextTable() {
        this.continue = true;
    }
    empty(table) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info('empty: ', table.name);
            yield this.dbConnector.emptyTable(table);
        });
    }
    getForeignKeyValues(table, tableForeignKeyValues = {}, runRows) {
        return __awaiter(this, void 0, void 0, function* () {
            for (var c = 0; c < table.columns.length; c++) {
                const column = table.columns[c];
                if (column.foreignKey) {
                    const foreignKey = column.foreignKey;
                    let values = yield this.dbConnector.getValuesForForeignKeys(table.name, column.name, column.foreignKey.table, column.foreignKey.column, runRows, column.options.unique, column.foreignKey.where);
                    if (values.length === 0 && !column.options.nullable) {
                        throw new Error(`${table.name}: Not enough values available for foreign key ${foreignKey.table}.${foreignKey.column}`);
                    }
                    tableForeignKeyValues[`${column.name}_${foreignKey.table}_${foreignKey.column}`] = values;
                }
            }
        });
    }
    fillTables(reset = false) {
        return __awaiter(this, void 0, void 0, function* () {
            this.beforeAll();
            /** @todo: Remove deprecated warning */
            let useDeprecatedLines = false;
            for (const table of this.schema.tables) {
                if (table.lines) {
                    useDeprecatedLines = true;
                    table.maxLines = table.lines;
                }
                yield this.fill(table, reset);
            }
            if (useDeprecatedLines)
                console.warn('DEPRECATED: Table.lines is deprecated, please use table.maxLines instead.');
            /****************/
            this.afterAll();
        });
    }
    fill(table, reset = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (reset)
                yield this.empty(table);
            this.logger.info('fill: ', table.name);
            let handleTriggers = table.disableTriggers || (table.disableTriggers === undefined && this.schema.settings.disableTriggers);
            if (handleTriggers)
                yield this.dbConnector.disableTriggers(table.name);
            yield this.before(table);
            yield this.generateData(table);
            yield this.after(table);
            if (handleTriggers)
                yield this.dbConnector.enableTriggers(table.name);
        });
    }
    beforeAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.schema.settings.beforeAll)
                return;
            for (const query of this.schema.settings.beforeAll) {
                yield this.dbConnector.executeRawQuery(query);
            }
        });
    }
    afterAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.schema.settings.afterAll)
                return;
            for (const query of this.schema.settings.afterAll) {
                yield this.dbConnector.executeRawQuery(query);
            }
        });
    }
    before(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!table.before)
                return;
            for (const query of table.before) {
                yield this.dbConnector.executeRawQuery(query);
            }
        });
    }
    generateData(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const tableForeignKeyValues = {};
            let previousRunRows = -1;
            let currentNbRows = yield this.dbConnector.countLines(table);
            let maxLines = 0;
            if (table.addLines) {
                maxLines = currentNbRows + table.addLines;
                if (table.maxLines !== undefined)
                    maxLines = Math.min(maxLines, table.maxLines);
            }
            else if (table.maxLines)
                maxLines = table.maxLines;
            process.stdout.write(currentNbRows + ' / ' + maxLines);
            batch: while (currentNbRows < maxLines) {
                previousRunRows = currentNbRows;
                const rows = [];
                const runRows = Math.min(1000, maxLines - currentNbRows);
                try {
                    yield this.getForeignKeyValues(table, tableForeignKeyValues, runRows);
                }
                catch (ex) {
                    process.stdout.write('\n');
                    this.logger.warn(ex.message);
                    break batch;
                }
                for (let i = 0; i < runRows; i++) {
                    const row = {};
                    for (var c = 0; c < table.columns.length; c++) {
                        const column = table.columns[c];
                        if (column.options.autoIncrement)
                            continue;
                        if (column.values) {
                            let parsedValues = [];
                            let values = column.values;
                            if (typeof values === 'string') {
                                values = this.schema.settings.values[values];
                            }
                            if (!(values instanceof Array)) {
                                Object.keys(values).forEach((key) => {
                                    let arr = new Array(Math.round(values[key] * 100));
                                    arr = arr.fill(key);
                                    parsedValues = parsedValues.concat(arr);
                                });
                            }
                            else {
                                parsedValues = values;
                            }
                            row[column.name] = this.random.pick(parsedValues);
                            continue;
                        }
                        if (column.foreignKey) {
                            const foreignKeys = tableForeignKeyValues[`${column.name}_${column.foreignKey.table}_${column.foreignKey.column}`];
                            if (foreignKeys.length > 0)
                                row[column.name] = foreignKeys[i % foreignKeys.length];
                            continue;
                        }
                        switch (column.generator) {
                            case 'set':
                            case 'bit':
                                row[column.name] = this.random.integer(0, Math.pow(2, column.options.max));
                                break;
                            case 'bool':
                            case 'boolean':
                                row[column.name] = this.random.bool();
                                break;
                            case 'smallint':
                            case 'mediumint':
                            case 'tinyint':
                            case 'int':
                            case 'integer':
                            case 'bigint':
                                row[column.name] = this.random.integer(column.options.min, column.options.max);
                                break;
                            case 'decimal':
                            case 'dec':
                            case 'float':
                            case 'double':
                                row[column.name] = this.random.real(column.options.min, column.options.max);
                                break;
                            case 'date':
                            case 'datetime':
                            case 'timestamp':
                                const min = column.options.min ? new Date(column.options.min) : new Date('01-01-1970');
                                const max = column.options.max ? new Date(column.options.max) : new Date();
                                row[column.name] = this.random.date(min, max);
                                break;
                            case 'time':
                                const hours = this.random.integer(-838, +838);
                                const minutes = this.random.integer(-0, +59);
                                const seconds = this.random.integer(-0, +59);
                                row[column.name] = `${hours}:${minutes}:${seconds}`;
                                break;
                            case 'year':
                                row[column.name] = this.random.integer(column.options.min, column.options.max);
                                break;
                            case 'varchar':
                            case 'char':
                            case 'binary':
                            case 'varbinary':
                            case 'tinytext':
                            case 'text':
                            case 'mediumtext':
                            case 'longtext':
                            case 'tinyblob':
                            case 'blob':
                            case 'mediumblob': // 16777215
                            case 'longblob': // 4,294,967,295
                                if (column.options.max >= 36 && column.options.unique) {
                                    row[column.name] = this.random.uuid4();
                                }
                                else {
                                    row[column.name] = this.random.string(this.random.integer(column.options.min, column.options.max));
                                }
                                break;
                            case 'enum':
                                row[column.name] = Math.floor(this.random.realZeroToOneExclusive() * (column.options.max)) + 1;
                                break;
                        }
                        if (column.options.nullable && this.random.realZeroToOneExclusive() <= 0.1)
                            row[column.name] = null;
                    }
                    rows.push(row);
                }
                currentNbRows += yield this.dbConnector.insert(table.name, rows);
                if (previousRunRows === currentNbRows) {
                    process.stdout.write('\n');
                    this.logger.warn(`Last run didn't insert any new rows in ${table.name}`);
                    break batch;
                }
                process.stdout.clearLine(-1);
                process.stdout.cursorTo(0);
                process.stdout.write(currentNbRows + ' / ' + maxLines);
                if (this.continue) {
                    this.continue = false;
                    break batch;
                }
            }
            process.stdout.write('\n');
        });
    }
    after(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!table.after)
                return;
            for (const query of table.after) {
                yield this.dbConnector.executeRawQuery(query);
            }
        });
    }
}
exports.Generator = Generator;
